"use client";

import React, { useEffect, useState } from "react";

type User = {
  id: number;
  first_name: string;
  last_name: string;
  email: string;
  gender: string;
  phone: string;
};

const Home = () => {
  const [data, setData] = useState<User[]>([]);
  const [deleteSuccessMessage, setDeleteSuccessMessage] = useState("");
  const [createSuccessMessage, setCreateSuccessMessage] = useState("");
  const [updateSuccessMessage, setUpdateSuccessMessage] = useState("");

  useEffect(() => {
    fetchData();
  }, []);

  const fetchData = async () => {
    try {
      const res = await fetch("http://localhost:10091/users");
      const responseData = await res.json();
      setData(responseData.info);
    } catch (err) {
      console.error(err);
    }
  };

  const setTimedMessage = (setMessage: (message: string) => void, message: string) => {
    setMessage(message);
    setTimeout(() => {
      setMessage("");
    }, 60000);
  };

  const handleDelete = async (id: number) => {
    try {
      await fetch(`http://localhost:10091/users/${id}`, { method: "DELETE" });
      setData(data.filter((user) => user.id !== id)); // Remove the deleted user from the state
      setTimedMessage(setDeleteSuccessMessage, "Delete successful");
    } catch (err) {
      console.error(err);
    }
  };

  const handleCreateOrUpdate = async (method: string, url: string, userData: User, setMessage: (message: string) => void) => {
    try {
      const res = await fetch(url, {
        method: method,
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify(userData),
      });
      setTimedMessage(setMessage, `User ${method === "POST" ? "created" : "updated"} successfully`);
      fetchData();
    } catch (err) {
      console.error(err);
    }
  };

  const handleCreate = async (userData: User) => {
    await handleCreateOrUpdate("POST", "http://localhost:10091/users", userData, setCreateSuccessMessage);
  };

  const handleUpdate = async (id: number, updatedUserData: User) => {
    await handleCreateOrUpdate("PUT", `http://localhost:10091/users/${id}`, updatedUserData, setUpdateSuccessMessage);
  };

  return (
    <main>
      <div>
        <h1>Next.js - HTTP CRUD example</h1>
        {deleteSuccessMessage && <div>{deleteSuccessMessage}</div>}
        {createSuccessMessage && <div>{createSuccessMessage}</div>}
        {updateSuccessMessage && <div>{updateSuccessMessage}</div>}
        <table>
          <thead>
            <tr>
              <th>first name</th>
              <th>last name</th>
              <th>email address</th>
              <th>gender</th>
              <th>phone number</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {data.map((user) => (
              <tr key={user.id}>
                <td>{user.first_name}</td>
                <td>{user.last_name}</td>
                <td>{user.email}</td>
                <td>{user.gender}</td>
                <td>{user.phone}</td>
                <td>
                  <button onClick={() => handleDelete(user.id)}>Delete</button>
                  <button onClick={() => handleUpdate(user.id, { ...user, firstName: "Updated" })}>Update</button>
                </td>
              </tr>
            ))}
          </tbody>
        </table>
        <button onClick={() => handleCreate({ firstName: "New", lastName: "User", email: "newuser@example.com", gender: "Male", phone: "1234567890" })}>Create New User</button>
      </div>
    </main>
  );
};

export default Home;
